unit fmUserList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, SvCom_SidCenter, ComCtrls, SvCom_AclApi;

type
    TUsersListForm = class(TForm)
        Label1: TLabel;
        EdDomain: TEdit;
        Timer: TTimer;
        SidC: TsvSidCenter;
        LV: TListView;
        Label2: TLabel;
        EdUserName: TEdit;
        LVSelected: TListView;
        BnAdd: TButton;
        BnDelete: TButton;
        BnOK: TButton;
        BnCancel: TButton;
        CBxAccessMode: TComboBox;
        Label3: TLabel;
        Label4: TLabel;
    BnEditAccessMask: TButton;
        MAccessMaskText: TMemo;
        procedure EdDomainChange(Sender: TObject);
        procedure TimerTimer(Sender: TObject);
        procedure EdUserNameChange(Sender: TObject);
        procedure LVClick(Sender: TObject);
        procedure BnAddClick(Sender: TObject);
        procedure LVSelectedChange(Sender: TObject; Item: TListItem;
          Change: TItemChange);
        procedure BnDeleteClick(Sender: TObject);
        procedure LVDblClick(Sender: TObject);
        procedure BnEditAccessMaskClick(Sender: TObject);
        procedure CBxAccessModeChange(Sender: TObject);
    private
        { Private declarations }
        _IsBusy: Boolean;
        _AccessMask: DWORD;
    public
        { Public declarations }
        property AccessMask: DWORD read _AccessMask;
        function AccessMode: TAccessMode;
        constructor Create(AComponent: TComponent); override;
    end;

var
  UsersListForm: TUsersListForm;

implementation

{$R *.DFM}

uses fmFileRights;

constructor TUsersListForm.Create(AComponent: TComponent);
begin
    inherited;
end;

procedure TUsersListForm.EdDomainChange(Sender: TObject);
begin
    Timer.Enabled:=True;
end;

procedure TUsersListForm.TimerTimer(Sender: TObject);
var wn : TsvWellKnownSid;
    LI: TListItem;
begin
    if _IsBusy then exit;
    _IsBusy:=True;
    try
        SidC.Domain:=EdDomain.Text;
        SidC.UserName:=EdUserName.Text;
//        LbSID.Caption:=SidC.SidString;

        LV.Items.Clear;
        for wn:=Low(TsvWellKnownSid) to High(TsvWellKnownSid) do
        begin
            if wn in [wkCurrentUser,wkOther] then continue;
            SidC.Domain:=EdDomain.Text;
            SidC.WellKnown:=wn;
            if SidC.UserName = '' then continue;
            LI:=LV.Items.Add;
            LI.Caption:=SidC.Domain+'\'+SidC.UserName;
            LI.Data:=SidC.GetSidCopy;
        end;
    finally
        _IsBusy:=False;
        Timer.Enabled:=False;
    end;
end;

procedure TUsersListForm.EdUserNameChange(Sender: TObject);
begin
    Timer.Enabled:=True;
end;

procedure TUsersListForm.LVClick(Sender: TObject);
begin
    if LV.Selected = nil then exit;
    SidC.SID:=LV.Selected.Data;
    EdDomain.Text:=SidC.Domain;
    EdUserName.Text:=SidC.UserName;
    Timer.Enabled:=False;
//    LbSID.Caption:=SidC.SidString;
end;

procedure TUsersListForm.BnAddClick(Sender: TObject);
var i,j: Integer;
    LIS,LID: TListItem;
begin
    if LV.SelCount > 1 then // Select items from the upper list view
    begin
        for i:=0 to LV.Items.Count-1 do
        begin
            LIS:=LV.Items[i];
            if not LIS.Selected then continue;  // We should add the selected items only

            LID:=LVSelected.Items.Add;
            LID.Caption:=LIS.Caption;
            LID.Data:=LIS.Data;
        end;
    end else // user is specified by name and domain
    begin
        LID:=LVSelected.Items.Add;
        LID.Caption:=SidC.Domain+'\'+SidC.UserName;
        SidC.Domain:=EdDomain.Text;
        SidC.UserName:=EdUserName.Text;
        LID.Data:=SidC.GetSidCopy;
    end;
end;

procedure TUsersListForm.LVSelectedChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
    BnDelete.Enabled:=LVSelected.SelCount > 0;
    BnOk.Enabled:=(LVSelected.Items.Count > 0) and (CBxAccessMode.Text <> '');
end;

procedure TUsersListForm.BnDeleteClick(Sender: TObject);
var L: TList;
    i: Integer;
    LI: TListItem;
begin
    L:=TList.Create;

    // At first collect all selected items into list
    for i:=0 to LVSelected.Items.Count-1 do
    begin
        LI:=LVSelected.Items[i];
        if not LI.Selected then continue;
        L.Add(LI);
    end;

    // Then delete all collected items
    for i:=0 to L.Count-1 do TObject(L[i]).Free;

    L.Free;
end;

procedure TUsersListForm.LVDblClick(Sender: TObject);
begin
    BnAdd.Click;
end;

procedure TUsersListForm.BnEditAccessMaskClick(Sender: TObject);
var AF: TFileAccessForm;
begin
    AF:=TFileAccessForm.Create(nil);
    try
        AF.AccessMask:=AccessMask;
        if AF.ShowModal <> mrOK then exit;
        _AccessMask:=AF.AccessMask;
        MAccessMaskText.Text:=AF.AccessMaskText;
    finally
        AF.Release;
    end;
end;

function TUsersListForm.AccessMode: TAccessMode;
begin
    case CBxAccessMode.Items.IndexOf(CBxAccessMode.Text) of
        0: result:=GRANT_ACCESS;
        1: result:=SET_ACCESS;
        2: result:=DENY_ACCESS;
        3: result:=REVOKE_ACCESS;
    else
        raise Exception.Create('Invalid access mode');
    end;
end;

procedure TUsersListForm.CBxAccessModeChange(Sender: TObject);
begin
    BnOk.Enabled:=(LVSelected.Items.Count > 0) and (CBxAccessMode.Text <> '');
end;

end.
